@@ -1,11 +1,8 @@
+import hashlib
 import json
-import os
 
 import tornadoredis
-import tornado.web
-import tornado.ioloop
-import tornado.gen
-import tornado.httpserver
+from tornado import web, ioloop, gen
 
 from insight_reloaded.insight_settings import (
     REDIS_HOST, REDIS_PORT, REDIS_DB, SENTRY_DSN, REDIS_QUEUE_KEYS,
@@ -20,10 +17,9 @@ except ImportError:
         print "SENTRY_DSN is defined but raven isn't installed."
 
 
-HERE = os.path.dirname(os.path.abspath(__file__))
 try:
-    with open(os.path.join(HERE, '../VERSION')) as f:
-        VERSION = f.readlines()[0].strip()
+    from insight_reloaded import __version__
+    VERSION = __version__
 except IOError:
     VERSION = 'N/A'
 
@@ -33,10 +29,11 @@ c = tornadoredis.Client(host=REDIS_HOST, port=REDIS_PORT, selected_db=REDIS_DB)
 c.connect()
 
 
-class MainHandler(tornado.web.RequestHandler):
+class MainHandler(web.RequestHandler):
     """Convert the querystring in a REDIS job JSON."""
 
-    @tornado.web.asynchronous
+    @web.asynchronous
+    @gen.coroutine
     def get(self, queue):
         # If there is no argument, display the version
         if self.request.arguments == {}:
@@ -50,7 +47,7 @@ class MainHandler(tornado.web.RequestHandler):
             queue = DEFAULT_REDIS_QUEUE_KEY
 
         if queue not in REDIS_QUEUE_KEYS:
-            raise tornado.web.HTTPError(404, 'Queue "%s" not found' % queue)
+            raise web.HTTPError(404, 'Queue "%s" not found' % queue)
 
         self.queue = queue
 
@@ -65,12 +62,13 @@ class MainHandler(tornado.web.RequestHandler):
                                                self.request.host,
                                                params['url'])
         else:
-            raise tornado.web.HTTPError(404, 'Input file not found')
+            raise web.HTTPError(404, 'Input file not found')
 
         # Max number of pages to compile
         try:
-            params['max_previews'] = int(self.get_argument('pages', MAX_PAGES_PREVIEW))
-        except:
+            params['max_previews'] = int(
+                self.get_argument('pages', MAX_PAGES_PREVIEW))
+        except ValueError:
             params['max_previews'] = MAX_PAGES_PREVIEW
 
         try:
@@ -78,37 +76,42 @@ class MainHandler(tornado.web.RequestHandler):
         except ValueError:
             params['crop'] = CROP_SIZE
 
+        try:
+            params['hash'] = self.get_argument('hash')
+        except web.MissingArgumentError:
+            params['hash'] = hashlib.sha1(params['url']).hexdigest()
+
         message = json.dumps(params)
-        c.rpush(self.queue, message, callback=self.on_response)
+        nb_jobs = yield gen.Task(c.rpush, self.queue, message)
 
-    def on_response(self, response):
         self.write(dict(insight_reloaded="Job added to queue '%s'." %
                         self.queue,
-                        number_in_queue=response))
+                        number_in_queue=nb_jobs))
         self.finish()
 
 
-class StatusHandler(tornado.web.RequestHandler):
-    @tornado.web.asynchronous
+class StatusHandler(web.RequestHandler):
+
+    @web.asynchronous
+    @gen.coroutine
     def get(self, queue):
         if not queue:
             queue = DEFAULT_REDIS_QUEUE_KEY
 
         if queue not in REDIS_QUEUE_KEYS:
-            raise tornado.web.HTTPError(404, "Queue '%s' not found" % queue)
+            raise web.HTTPError(404, "Queue '%s' not found" % queue)
         self.queue = queue
-        c.llen(self.queue, callback=self.on_response)
+        nb_jobs = yield gen.Task(c.llen, self.queue)
 
-    def on_response(self, response):
         self.write(dict(insight_reloaded="There is %d job in the '%s' queue." %
-                        (response, self.queue),
-                        number_in_queue=response))
+                        (nb_jobs, self.queue),
+                        number_in_queue=nb_jobs))
         self.finish()
 
 
-application = tornado.web.Application([
-    (r"/([A-Za-z0-9_-]*)/?status", StatusHandler),
-    (r"/([A-Za-z0-9_-]*)", MainHandler),
+application = web.Application([
+    (r"^/?([A-Za-z0-9_-]*)/status$", StatusHandler),  # /status must be valid
+    (r"^/([A-Za-z0-9_-]*)$", MainHandler),
 ])
 
 
@@ -120,7 +123,7 @@ def main():
     print "Launch tornado ioloop"
     application.listen(8888)
     try:
-        tornado.ioloop.IOLoop.instance().start()
+        ioloop.IOLoop.instance().start()
     except Exception:
         if client:
             client.get_ident(client.captureException())
