@@ -1,18 +1,19 @@
 from os.path import join
-import hashlib
 
 import pyrax
 pyrax.set_setting("identity_type", "rackspace")
 
-from insight_reloaded.insight_settings import (CLOUDFILES_USERNAME,
-    CLOUDFILES_API_KEY, CLOUDFILES_COUNTAINER)
+from insight_reloaded.insight_settings import (
+    CLOUDFILES_USERNAME, CLOUDFILES_API_KEY, CLOUDFILES_COUNTAINER,
+    CLOUDFILES_SERVICENET
+)
 
 
 class CloudFilesStorage(object):
 
-    def __init__(self, document_url, cloudfiles_location=None):
+    def __init__(self, document_url, document_hash, cloudfiles_location=None):
         self.document_url = document_url
-        self.document_hash = hashlib.sha1(self.document_url).hexdigest()
+        self.document_hash = document_hash
 
         if cloudfiles_location is None:
             cloudfiles_location = "LON"
@@ -21,7 +22,10 @@ class CloudFilesStorage(object):
         pyrax.set_credentials(CLOUDFILES_USERNAME, CLOUDFILES_API_KEY)
         self.paths = {}
 
-        self.container = pyrax.cloudfiles.create_container(CLOUDFILES_COUNTAINER)
+        self.client = pyrax.connect_to_cloudfiles(
+            cloudfiles_location, public=not CLOUDFILES_SERVICENET)
+        self.container = self.client.cloudfiles.create_container(
+            CLOUDFILES_COUNTAINER)
 
     def prepare(self):
         """Do nothing
